package ittp.protocol.request;

import ittp.ittpException;
import ittp.protocol.Protocol;
import ittp.protocol.response.Response;
import ittp.protocol.request.headers.*;
import java.util.HashMap;
import java.net.Socket;
import java.io.File;
import java.rmi.RemoteException;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * This class build the request send by the client.
 * @author Frederic Bidon and Mathieu Texier
 */
public abstract class Request extends Protocol{
    /**Declaration of the different headers send in a request*/
    static final public String CONNECTION = "Connection";
    static final public String DATE = "Date";
    static final public String HOST = "Host";
    static final public String MODIFIED_SINCE = "IF-Modified-Since";
    static final public String RANGE = "Range";
    /**Attribute a value to each method for differenciate them.*/
    static final public int GET = 0;
    static final public int HEAD = 1;
    static final public int OPTIONS = 2;
    static final public int TRACE = 3;
    /**Creation of a string array containing the different method that can be used.*/
    static final public String[] methodName = new String[] {"GET", "HEAD", "OPTIONS", "TRACE"};
    
    protected int Id;
    private File file;
    private HashMap headerList;
    private int port = 0;
    private String sheme = new String();
    private String host = new String();
    private String RelativePath = new String();
    private InetAddress ip;
    private InetAddress clientIP;
    
    public Request() {
        headerList = new HashMap();
        headerList.put(CONNECTION, new Connection(this));
        headerList.put(DATE, new Date(this));
        headerList.put(HOST, new Host(this));
        headerList.put(MODIFIED_SINCE, new ModifiedSince(this));
        headerList.put(RANGE, new Range(this));
    }
    
    abstract public void process(Response response);
    
    /**
     * Accessor for the object Headers of request
     * @param headerString String containing the name of the header.
     * @return an object Headers correponding to the name headerstring.
     */
    public Headers getHeaders(String headerString ) {
        return (Headers) headerList.get(headerString);
    }
    
    /**
     * Accessors that allows to return the name of file requested by the client.
     * @return the file.
     */
    public File getFile() {
        return file;
    }
    
    /**
     * Accessors that allows to return the IP requested by the client.
     * @return an InetAddress ip.
     */
    public InetAddress getIP() {
        return clientIP;
    }
    
    /**
     * Mutators for the ClientIP.
     * @param clientIP InetAddress that contain the IP address.
     */
    public void setIP(InetAddress clientIP) {
        this.clientIP = clientIP;
    }
    
    /**
     * Mutators for the String sheme.
     * @param sheme String that contain the protocol use to communicate.
     */
    public void setSheme(String sheme) {
        this.sheme = sheme;
    }
    
    public InetAddress toIP(String host) {
        String[] part = host.split("\\.");
        System.out.println(" HOST :" + host);
        if (part.length == 4) {
            byte[] addr = new byte[4] ;
            try {
                for (int i=0; i<4; i++)
                addr[i] =  Byte.parseByte(part[i]);
                InetAddress inetAddress = InetAddress.getByAddress(addr);
                System.out.println(host + " " + inetAddress);
                return inetAddress;
            } catch (NumberFormatException e) {
            } catch (UnknownHostException e) {
            }
        }
        try {
            return InetAddress.getByName(host);
        } catch (UnknownHostException e){;}
        return null;
    }
    
    /**
     * Mutators for the String host.
     * @param host String that the name of the host requested by the client.
     */
    public void setHost(String host) {
        this.host = host;
        InetAddress ip = toIP(host);
        if (ip == null)
            throw new ittpException("nonvalide IP address or host " + ittp.Server.getIP(), Response.Code.ITTP_NOT_FOUND);
        if (!ip.equals(ittp.Server.getIP()) && !host.toLowerCase ().equals("localhost"))
            throw new ittpException("IP " + ip + "address does not match" + ittp.Server.getIP(), Response.Code.ITTP_NOT_FOUND);
        this.ip = ip;
    }
    
    /**
     * Mutators for the int port.
     * @param port int that determine the number of port used to communicate.
     */
    public void setPort(int port) {
        try {
            if (port != ittp.Server.getParameters().getPort())
                throw new ittpException("Port does not match", Response.Code.ITTP_NOT_FOUND);
            else {this.port = port;}
        } catch (RemoteException e) {throw new ittpException(e.getMessage(), Response.Code.ITTP_INTERNAL_ERROR);}
    }
    
    /**
     * Mutators for the String RelativePath.
     * @param RelativePath String that determines relative path.
     */
    public void setPath(String RelativePath) {
        String path = new String();
        try {
            path = ittp.Server.getParameters().getBaseDir() + RelativePath ;
        } catch (RemoteException e) {
            throw new ittpException(e.getMessage(), Response.Code.ITTP_INTERNAL_ERROR);
        }
        File file = new File(path);
        if (!file.canRead() || !file.exists())
            throw new ittpException("Cannot read the file: " + file.getPath(), Response.Code.ITTP_NOT_FOUND);
        this.RelativePath = RelativePath;
        this.file = file;
    }
    
    /**
     * Convert to a string the headers request.
     * @return a string containing the request of the client.
     */
    public String toString() {
        String retString  = new String();
        retString += methodName[Id] + " ";
        if (sheme.length() > 0)
            retString += sheme+":";
        retString += "/";
        if (host.length() > 0) {
            retString += "/" + host;
            if (port > 0) {
                retString += ":" + port;
            }
            retString += "/";
        }

        retString += RelativePath;
        retString += " " + ITTP_version;
        retString += "\n";
        retString += getHeaders(Request.CONNECTION);
        retString += getHeaders(Request.DATE);
        retString += getHeaders(Request.HOST);
        retString += getHeaders(Request.MODIFIED_SINCE);
        retString += getHeaders(Request.RANGE);
        retString += "\n";
        return retString;
    }
    
    /**
     * Abstract class headers that allows to build the different headers for the request.
     */
    abstract public class Headers {
        
        abstract public boolean isUsed();
        abstract public String toString();
    }
}
